/* MIPS specific symbolic name handling.
   Copyright (C) 2002, 2003, 2005 Red Hat, Inc.
   This file is part of Red Hat elfutils.
   Written by Jakub Jelinek <jakub@redhat.com>, 2002.

   Red Hat elfutils is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by the
   Free Software Foundation; version 2 of the License.

   Red Hat elfutils is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with Red Hat elfutils; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301 USA.

   Red Hat elfutils is an included package of the Open Invention Network.
   An included package of the Open Invention Network is a package for which
   Open Invention Network licensees cross-license their patents.  No patent
   license is granted, either expressly or impliedly, by designation as an
   included package.  Should you wish to participate in the Open Invention
   Network licensing program, please visit www.openinventionnetwork.com
   <http://www.openinventionnetwork.com>.  */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <elf.h>
#include <stddef.h>
#include <stdio.h>

#define BACKEND		mips_
#include "libebl_CPU.h"

/* Check for the simple reloc types.  */
Elf_Type
mips_reloc_simple_type (Ebl *ebl __attribute__ ((unused)), int type)
{
  switch (type)
    {
    case R_MIPS_16:
      return ELF_T_HALF;
    case R_MIPS_32:
      return ELF_T_WORD;
    case R_MIPS_64:
      return ELF_T_XWORD;
    default:
      return ELF_T_NUM;
    }
}
bool
mips_machine_flag_check (GElf_Word flags)
{
	return ((flags & ~(0xff | EF_MIPS_ARCH_ASE | EF_MIPS_ARCH | EF_MIPS_32BITMODE | EF_MIPS_ABI)) == 0);
}

const char *
mips_machine_flag_name (Elf64_Word *flags)
{

	if (*flags & EF_MIPS_NOREORDER) {
		*flags &= ~EF_MIPS_NOREORDER;
		return ".noreorder";
	}

	if (*flags & EF_MIPS_PIC) {
		*flags &= ~EF_MIPS_PIC;
		return "pic";
	}

	if (*flags & EF_MIPS_CPIC) {
		*flags &= ~EF_MIPS_CPIC;
		return "cpic";
	}

	if (*flags & EF_MIPS_XGOT) {
		*flags &= ~EF_MIPS_XGOT;
		return "xgot";
	}

	if (*flags & EF_MIPS_64BIT_WHIRL) {
		*flags &= ~EF_MIPS_64BIT_WHIRL;
		return "ugen_reserved";
	}

	if (*flags & EF_MIPS_ABI2) {
		*flags &= ~EF_MIPS_ABI2;
		return "abi2";
	}

	if (*flags & EF_MIPS_ABI_ON32) {
		*flags &= ~EF_MIPS_ABI_ON32;
		return "n32";
	}

	if (*flags & EF_MIPS_OPTIONS_FIRST) {
		*flags &= ~EF_MIPS_OPTIONS_FIRST;
		return "odk first";
	}

	if (*flags & EF_MIPS_32BITMODE) {
		*flags &= ~EF_MIPS_32BITMODE;
		return "32bitmode";
	}

	/* EF_MIPS_ABI is a gnu extension. */
	switch (*flags & EF_MIPS_ABI) {
		case E_MIPS_ABI_O32:
			*flags &= ~EF_MIPS_ABI;
			return "o32";
			break;
		case E_MIPS_ABI_O64:
			*flags &= ~EF_MIPS_ABI;
			return "o64";
			break;
		case E_MIPS_ABI_EABI32:
			*flags &= ~EF_MIPS_ABI;
			return "eabi32";
			break;
		case E_MIPS_ABI_EABI64:
			*flags &= ~EF_MIPS_ABI;
			return "eabi64";
			break;
		default:	break;
	}

	switch (*flags & EF_MIPS_ARCH_ASE) {
		case EF_MIPS_ARCH_ASE_MDMX:
			*flags &= ~EF_MIPS_ARCH_ASE;
			return "mdmx";
			break;
		case EF_MIPS_ARCH_ASE_M16:
			*flags &= ~EF_MIPS_ARCH_ASE;
			return "isa";
			break;
		default:
			break;
	}

	switch (*flags & EF_MIPS_ARCH) {
		case EF_MIPS_ARCH_1:
			/* never reached in pratice */
			*flags &= ~EF_MIPS_ARCH;
			return "mips1";
			break;
		case EF_MIPS_ARCH_2:
			*flags &= ~EF_MIPS_ARCH;
			return "mips2";
			break;
		case EF_MIPS_ARCH_3:
			*flags &= ~EF_MIPS_ARCH;
			return "mips3";
			break;
		case EF_MIPS_ARCH_4:
			*flags &= ~EF_MIPS_ARCH;
			return "mips4";
			break;
		case EF_MIPS_ARCH_5:
			*flags &= ~EF_MIPS_ARCH;
			return "mips5";
			break;
		case EF_MIPS_ARCH_32:
			*flags &= ~EF_MIPS_ARCH;
			return "mips32";
			break;
		case EF_MIPS_ARCH_64:
			*flags &= ~EF_MIPS_ARCH;
			return "mips64";
			break;
		default:
			break;
	}

	return NULL;
}

const char *
mips_segment_type_name (int segment, char *buf __attribute__ ((unused)),
			size_t len __attribute__ ((unused)))
{
  switch (segment)
    {
	case PT_MIPS_REGINFO:
	     return "REGINFO";
	     break;
	case PT_MIPS_RTPROC:
	     return "RTPROC";
	     break;
	case PT_MIPS_OPTIONS:
	     return "OPTIONS";
	    break;
    default:
      break;
    }
  return NULL;
}

const char *
mips_section_type_name (int type,
			char *buf __attribute__ ((unused)),
			size_t len __attribute__ ((unused)))
{
  switch (type)
    {
	case SHT_MIPS_LIBLIST:		 return "SHT_MIPS_LIBLIST";
	case SHT_MIPS_MSYM:		 return "SHT_MIPS_MSYM";
	case SHT_MIPS_CONFLICT:		 return "SHT_MIPS_CONFLICT";
	case SHT_MIPS_GPTAB:		 return "SHT_MIPS_GPTAB";
	case SHT_MIPS_UCODE:		 return "SHT_MIPS_UCODE";
	case SHT_MIPS_DEBUG:		 return "SHT_MIPS_DEBUG";
	case SHT_MIPS_REGINFO:		 return "SHT_MIPS_REGINFO";
	case SHT_MIPS_PACKAGE:		 return "SHT_MIPS_PACKAGE";
	case SHT_MIPS_PACKSYM:		 return "SHT_MIPS_PACKSYM";
	case SHT_MIPS_RELD:		 return "SHT_MIPS_RELD";
	case SHT_MIPS_IFACE:		 return "SHT_MIPS_IFACE";
	case SHT_MIPS_CONTENT:		 return "SHT_MIPS_CONTENT";
	case SHT_MIPS_OPTIONS:		 return "SHT_MIPS_OPTIONS";
	case SHT_MIPS_SHDR:		 return "SHT_MIPS_SHDR";
	case SHT_MIPS_FDESC:		 return "SHT_MIPS_FDESC";
	case SHT_MIPS_EXTSYM:		 return "SHT_MIPS_EXTSYM";
	case SHT_MIPS_DENSE:		 return "SHT_MIPS_DENSE";
	case SHT_MIPS_PDESC:		 return "SHT_MIPS_PDESC";
	case SHT_MIPS_LOCSYM:		 return "SHT_MIPS_LOCSYM";
	case SHT_MIPS_AUXSYM:		 return "SHT_MIPS_AUXSYM";
	case SHT_MIPS_OPTSYM:		 return "SHT_MIPS_OPTSYM";
	case SHT_MIPS_LOCSTR:		 return "SHT_MIPS_LOCSTR";
	case SHT_MIPS_LINE:		 return "SHT_MIPS_LINE";
	case SHT_MIPS_RFDESC:		 return "SHT_MIPS_RFDESC";
	case SHT_MIPS_DELTASYM:		 return "SHT_MIPS_DELTASYM";
	case SHT_MIPS_DELTAINST:	 return "SHT_MIPS_DELTAINST";
	case SHT_MIPS_DELTACLASS:	 return "SHT_MIPS_DELTACLASS";
	case SHT_MIPS_DWARF:		 return "SHT_MIPS_DWARF";
	case SHT_MIPS_DELTADECL:	 return "SHT_MIPS_DELTADECL";
	case SHT_MIPS_SYMBOL_LIB:	 return "SHT_MIPS_SYMBOL_LIB";
	case SHT_MIPS_EVENTS:		 return "SHT_MIPS_EVENTS";
	case SHT_MIPS_TRANSLATE:	 return "SHT_MIPS_TRANSLATE";
	case SHT_MIPS_PIXIE:		 return "SHT_MIPS_PIXIE";
	case SHT_MIPS_XLATE:		 return "SHT_MIPS_XLATE";
	case SHT_MIPS_XLATE_DEBUG:	 return "SHT_MIPS_XLATE_DEBUG";
	case SHT_MIPS_WHIRL:		 return "SHT_MIPS_WHIRL";
	case SHT_MIPS_EH_REGION:	 return "SHT_MIPS_EH_REGION";
	case SHT_MIPS_XLATE_OLD:	 return "SHT_MIPS_XLATE_OLD";
	case SHT_MIPS_PDR_EXCEPTION:	 return "SHT_MIPS_PDR_EXCEPTION";
	default:			break;
    }

  return NULL;
}
const char *
mips_dynamic_tag_name (int64_t tag, char *buf __attribute__ ((unused)),
			size_t len __attribute__ ((unused)))
{
	switch (tag) {
		case DT_MIPS_RLD_VERSION:		 return "DT_MIPS_RLD_VERSION";
		case DT_MIPS_TIME_STAMP:		 return "DT_MIPS_TIME_STAMP";
		case DT_MIPS_ICHECKSUM:			 return "DT_MIPS_ICHECKSUM";
		case DT_MIPS_IVERSION:			 return "DT_MIPS_IVERSION";
		case DT_MIPS_FLAGS:			 return "DT_MIPS_FLAGS";
		case DT_MIPS_BASE_ADDRESS:		 return "DT_MIPS_BASE_ADDRESS";
		case DT_MIPS_MSYM:			 return "DT_MIPS_MSYM";
		case DT_MIPS_CONFLICT:			 return "DT_MIPS_CONFLICT";
		case DT_MIPS_LIBLIST:			 return "DT_MIPS_LIBLIST";
		case DT_MIPS_LOCAL_GOTNO:		 return "DT_MIPS_LOCAL_GOTNO";
		case DT_MIPS_CONFLICTNO:		 return "DT_MIPS_CONFLICTNO";
		case DT_MIPS_LIBLISTNO:			 return "DT_MIPS_LIBLISTNO";
		case DT_MIPS_SYMTABNO:			 return "DT_MIPS_SYMTABNO";
		case DT_MIPS_UNREFEXTNO:		 return "DT_MIPS_UNREFEXTNO";
		case DT_MIPS_GOTSYM:			 return "DT_MIPS_GOTSYM";
		case DT_MIPS_HIPAGENO:			 return "DT_MIPS_HIPAGENO";
		case DT_MIPS_RLD_MAP:			 return "DT_MIPS_RLD_MAP";
		case DT_MIPS_DELTA_CLASS:		 return "DT_MIPS_DELTA_CLASS";
		case DT_MIPS_DELTA_CLASS_NO:		 return "DT_MIPS_DELTA_CLASS_NO";
		case DT_MIPS_DELTA_INSTANCE:		 return "DT_MIPS_DELTA_INSTANCE";
		case DT_MIPS_DELTA_INSTANCE_NO:		 return "DT_MIPS_DELTA_INSTANCE_NO";
		case DT_MIPS_DELTA_RELOC:		 return "DT_MIPS_DELTA_RELOC";
		case DT_MIPS_DELTA_RELOC_NO:		 return "DT_MIPS_DELTA_RELOC_NO";
		case DT_MIPS_DELTA_SYM:			 return "DT_MIPS_DELTA_SYM";
		case DT_MIPS_DELTA_SYM_NO:		 return "DT_MIPS_DELTA_SYM_NO";
		case DT_MIPS_DELTA_CLASSSYM:		 return "DT_MIPS_DELTA_CLASSSYM";
		case DT_MIPS_DELTA_CLASSSYM_NO:		 return "DT_MIPS_DELTA_CLASSSYM_NO";
		case DT_MIPS_CXX_FLAGS:			 return "DT_MIPS_CXX_FLAGS";
		case DT_MIPS_PIXIE_INIT:		 return "DT_MIPS_PIXIE_INIT";
		case DT_MIPS_SYMBOL_LIB:		 return "DT_MIPS_SYMBOL_LIB";
		case DT_MIPS_LOCALPAGE_GOTIDX:		 return "DT_MIPS_LOCALPAGE_GOTIDX";
		case DT_MIPS_LOCAL_GOTIDX:		 return "DT_MIPS_LOCAL_GOTIDX";
		case DT_MIPS_HIDDEN_GOTIDX:		 return "DT_MIPS_HIDDEN_GOTIDX";
		case DT_MIPS_PROTECTED_GOTIDX:		 return "DT_MIPS_PROTECTED_GOTIDX";
		case DT_MIPS_OPTIONS:			 return "DT_MIPS_OPTIONS";
		case DT_MIPS_INTERFACE:			 return "DT_MIPS_INTERFACE";
		case DT_MIPS_DYNSTR_ALIGN:		 return "DT_MIPS_DYNSTR_ALIGN";
		case DT_MIPS_INTERFACE_SIZE:		 return "DT_MIPS_INTERFACE_SIZE";
		case DT_MIPS_RLD_TEXT_RESOLVE_ADDR:	 return "DT_MIPS_RLD_TEXT_RESOLVE_ADDR";
		case DT_MIPS_PERF_SUFFIX:		 return "DT_MIPS_PERF_SUFFIX";
		case DT_MIPS_COMPACT_SIZE:		 return "DT_MIPS_COMPACT_SIZE";
		case DT_MIPS_GP_VALUE:			 return "DT_MIPS_GP_VALUE";
		case DT_MIPS_AUX_DYNAMIC:		 return "DT_MIPS_AUX_DYNAMIC";
		default:				break;
	}
	return NULL;
}
bool
mips_dynamic_tag_check (int64_t tag)
{
	  return (tag >= DT_LOPROC && tag <= DT_LOPROC + DT_MIPS_NUM);
}
